:root {
    --background-image: url('/api/panorama');
    --background-overlay-color: rgba(100, 0, 0, 0.5);
    --text-color: #fff;
    --container-background-color: rgba(50, 0, 0, 0.8);
    --border-color: #ff0000;
    --header-text-color: #fff;
    --monitor-output-background-color: #200;
    --monitor-output-text-color: #fff;
    --monitor-output-border-color: #ff0000;
    --splash-text-color: #fff;

    /* Buttons */
    --button-background-color: #900;
    --button-border-color: #ff0000;
    --button-text-color: #fff;
    --button-hover-background-color: #b00;
    --button-hover-border-color: #ff0000;
    --button-hover-text-color: #fff;
    --button-active-background-color: #d00;
    --button-active-border-color: #ff0000;

    --primary-button-background-color: #ff0000;
    --primary-button-border-color: #ff0000;
    --primary-button-text-color: #fff;
    --primary-button-hover-background-color: #ff3333;
    --primary-button-hover-border-color: #ff0000;
    --primary-button-active-background-color: #ff6666;
    --primary-button-active-border-color: #ff0000;

    --danger-button-background-color: #ff0000;
    --danger-button-border-color: #ff0000;
    --danger-button-text-color: #fff;
    --danger-button-hover-background-color: #ff3333;
    --danger-button-hover-border-color: #ff0000;
    --danger-button-active-background-color: #ff6666;
    --danger-button-active-border-color: #ff0000;

    /* Forms */
    --form-input-background-color: #200;
    --form-input-border-color: #ff0000;
    --form-input-text-color: #fff;
    --form-input-focus-border-color: #ff3333;
    --form-label-text-color: #fff;
    --validation-error-text-color: #ff9999;
    --form-input-invalid-border-color: #ff9999;
    --form-input-invalid-background-color: #300;

    /* Tables */
    --table-header-background-color: #200;
    --table-header-text-color: #fff;
    --table-border-color: #ff0000;
    --table-row-hover-background-color: #400;

    /* Tabs */
    --tab-background-color: #200;
    --tab-border-color: #ff0000;
    --tab-text-color: #fff;
    --tab-hover-background-color: #400;
    --tab-hover-text-color: #fff;
    --tab-active-background-color: #600;
    --tab-active-text-color: #fff;

    /* Messages */
    --message-success-background-color: #020;
    --message-success-border-color: #0f0;
    --message-success-text-color: #9f9;
    --message-error-background-color: #200;
    --message-error-border-color: #f00;
    --message-error-text-color: #f99;
    --message-warning-background-color: #220;
    --message-warning-border-color: #ff0;
    --message-warning-text-color: #ff9;

    /* Layout */
    --sidebar-background-color: #200;
    --sidebar-border-color: #ff0000;
    --sidebar-link-text-color: #fff;
    --sidebar-link-hover-background-color: #400;
    --sidebar-link-hover-text-color: #fff;
    --sidebar-link-active-background-color: #600;
    --sidebar-link-active-border-color: #ff0000;
    --server-card-background-color: #200;
    --server-card-border-color: #ff0000;
    --server-card-hover-background-color: #400;
    --server-card-info-text-color: #ccc;
    --server-card-info-label-color: #fff;

    /* Controls */
    --toggle-switch-off-background-color: #200;
    --toggle-switch-off-border-color: #ff0000;
    --toggle-switch-off-handle-background-color: #ff0000;
    --toggle-switch-off-handle-border-color: #200;
    --toggle-switch-on-background-color: #ff0000;
    --toggle-switch-on-border-color: #200;
    --toggle-switch-on-handle-background-color: #200;
    --toggle-switch-on-handle-border-color: #ff0000;
}
