{% extends "base.html" %}

{% block title %}{{ super() }} - Content Uploader{% endblock %}

{% block content %}
<div class="content-section active">
    <h2>Upload Minecraft Content</h2>
    <p>Select a .mcworld, .mcpack, or .mcaddon file to upload.</p>

    <form action="{{ request.url_for('handle_file_upload') }}" method="post" enctype="multipart/form-data" class="form-grid">
        <div class="form-group">
            <label for="file" class="form-label">Choose file:</label>
            <input type="file" id="file" name="file" accept=".mcworld,.mcpack,.mcaddon" required class="form-input">
        </div>
        <div class="form-actions">
            <button type="submit" class="action-button start-button">Upload File</button>
        </div>
    </form>
    <div class="page-actions">
        <a href="{{ request.url_for('index') }}" class="action-link">« Back to Dashboard</a>
    </div>
</div>
{% endblock %}
