{% extends "base.html" %}

{% block title %}Bedrock Server Manager - Setup{% endblock %}

{% block content %}
<div class="container">
    <h1>Welcome to Bedrock Server Manager</h1>
    <p>Please create an admin account to get started.</p>
    <form id="setup-form">
        <div class="form-group">
            <label for="username">Username</label>
            <input type="text" id="username" name="username" class="form-input" required autocomplete="username">
        </div>
        <div class="form-group">
            <label for="password">Password</label>
            <input type="password" id="password" name="password" class="form-input" required autocomplete="new-password">
        </div>
        <div class="form-group">
            <label for="confirm_password">Confirm Password</label>
            <input type="password" id="confirm_password" name="confirm_password" class="form-input" required autocomplete="new-password">
        </div>
        <button type="submit" class="start-button">Create Admin User</button>
    </form>
</div>
{% endblock %}

{% block body_scripts %}
<script src="{{ request.url_for('static', path='js/setup.js') }}" defer></script>
{% endblock %}
