:root {
    --background-image: url('/api/panorama');
    --background-overlay-color: rgba(34, 34, 34, 0.25);
    --text-color: #eee;
    --container-background-color: rgba(51, 51, 51, 0.92);
    --border-color: #555;
    --header-text-color: #eee;
    --monitor-output-background-color: #111;
    --monitor-output-text-color: #00dd00;
    --monitor-output-border-color: #444;
    --splash-text-color: #FFFF00;
}
