# bedrock_server_manager/cli/__init__.py
"""
Bedrock Server Manager CLI Package.

This package contains the command-line interface modules for managing the Bedrock server.
It includes subcommands for various functionalities like server management, add-on handling,
backups, and more.
"""
