:root {
    --background-image: url('/api/panorama');
    --background-overlay-color: rgba(255, 255, 255, 0.25);
    --text-color: #000;
    --container-background-color: rgba(255, 255, 255, 0.92);
    --border-color: #ccc;
    --header-text-color: #000;
    --monitor-output-background-color: #eee;
    --monitor-output-text-color: #333;
    --monitor-output-border-color: #ccc;
    --splash-text-color: #FF0000;

    /* Buttons */
    --button-background-color: #f0f0f0;
    --button-border-color: #ccc;
    --button-text-color: #333;
    --button-hover-background-color: #e0e0e0;
    --button-hover-border-color: #bbb;
    --button-hover-text-color: #333;
    --button-active-background-color: #d0d0d0;
    --button-active-border-color: #aaa;

    --primary-button-background-color: #4CAF50;
    --primary-button-border-color: #66BB6A;
    --primary-button-text-color: #fff;
    --primary-button-hover-background-color: #6fcf97;
    --primary-button-hover-border-color: #81e0aa;
    --primary-button-active-background-color: #388E3C;
    --primary-button-active-border-color: #4CAF50;

    --danger-button-background-color: #D32F2F;
    --danger-button-border-color: #E57373;
    --danger-button-text-color: #fff;
    --danger-button-hover-background-color: #E53935;
    --danger-button-hover-border-color: #EF9A9A;
    --danger-button-active-background-color: #B71C1C;
    --danger-button-active-border-color: #D32F2F;

    /* Forms */
    --form-input-background-color: #fff;
    --form-input-border-color: #ccc;
    --form-input-text-color: #333;
    --form-input-focus-border-color: #aaa;
    --form-label-text-color: #333;
    --validation-error-text-color: #D32F2F;
    --form-input-invalid-border-color: #D32F2F;
    --form-input-invalid-background-color: #FFEBEE;

    /* Tables */
    --table-header-background-color: #f0f0f0;
    --table-header-text-color: #333;
    --table-border-color: #ccc;
    --table-row-hover-background-color: #f5f5f5;

    /* Tabs */
    --tab-background-color: #f0f0f0;
    --tab-border-color: #ccc;
    --tab-text-color: #333;
    --tab-hover-background-color: #e0e0e0;
    --tab-hover-text-color: #333;
    --tab-active-background-color: #fff;
    --tab-active-text-color: #333;

    /* Messages */
    --message-success-background-color: #E8F5E9;
    --message-success-border-color: #4CAF50;
    --message-success-text-color: #388E3C;
    --message-error-background-color: #FFEBEE;
    --message-error-border-color: #D32F2F;
    --message-error-text-color: #b71c1c;
    --message-warning-background-color: #FFF8E1;
    --message-warning-border-color: #FFA000;
    --message-warning-text-color: #212121;

    /* Layout */
    --sidebar-background-color: #f5f5f5;
    --sidebar-border-color: #ccc;
    --sidebar-link-text-color: #333;
    --sidebar-link-hover-background-color: #e0e0e0;
    --sidebar-link-hover-text-color: #333;
    --sidebar-link-active-background-color: #e0e0e0;
    --sidebar-link-active-border-color: #4CAF50;
    --server-card-background-color: #fff;
    --server-card-border-color: #ccc;
    --server-card-hover-background-color: #f5f5f5;
    --server-card-info-text-color: #000;
    --server-card-info-label-color: #000;

    /* Controls */
    --toggle-switch-off-background-color: #ccc;
    --toggle-switch-off-border-color: #bbb;
    --toggle-switch-off-handle-background-color: #fff;
    --toggle-switch-off-handle-border-color: #ccc;
    --toggle-switch-on-background-color: #4CAF50;
    --toggle-switch-on-border-color: #66BB6A;
    --toggle-switch-on-handle-background-color: #fff;
    --toggle-switch-on-handle-border-color: #eee;
}
